/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.BufferSyncerDto
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.MsgId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BufferSyncer", side = ProtocolSide.CORE)
interface BufferSyncerClientApi {
  @RpcCall("markBufferAsRead")
  suspend fun markBufferAsRead( @RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("mergeBuffersPermanently")
  suspend fun mergeBuffersPermanently(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.UserType.BufferId buffer2: BufferId)

  @RpcCall("removeBuffer")
  suspend fun removeBuffer(@RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("renameBuffer")
  suspend fun renameBuffer(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.QString newName: String)

  @RpcCall("setMarkerLine")
  suspend fun setMarkerLine(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.UserType.MsgId msgId: MsgId)

  @RpcCall("setLastSeenMsg")
  suspend fun setLastSeenMsg(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.UserType.MsgId msgId: MsgId)

  @RpcCall("setBufferActivity")
  suspend fun setBufferActivity(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int types: Int)

  @RpcCall("setHighlightCount")
  suspend fun setHighlightCount(@RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int count: Int)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: BufferSyncerDto) = update(properties.let(BufferSyncerDto.Serializer::serialize))
}
