/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client


import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("BufferViewConfig", side = ProtocolSide.CORE)
interface BufferViewConfigClientApi {
  @RpcCall("addBuffer")
  suspend fun addBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("moveBuffer")
  suspend fun moveBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId, @RpcParam.Int pos: Int)

  @RpcCall("removeBuffer")
  suspend fun removeBuffer(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("removeBufferPermanently")
  suspend fun removeBufferPermanently(objectName: ObjectName, @RpcParam.UserType.BufferId buffer: BufferId)

  @RpcCall("setBufferViewName")
  suspend fun setBufferViewName(objectName: ObjectName, @RpcParam.QString value: String)

  @RpcCall("setAddNewBuffersAutomatically")
  suspend fun setAddNewBuffersAutomatically(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("setAllowedBufferTypes")
  suspend fun setAllowedBufferTypes(objectName: ObjectName, @RpcParam.Int value: Int)

  @RpcCall("setDisableDecoration")
  suspend fun setDisableDecoration(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("setHideInactiveBuffers")
  suspend fun setHideInactiveBuffers(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("setHideInactiveNetworks")
  suspend fun setHideInactiveNetworks(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("setMinimumActivity")
  suspend fun setMinimumActivity(objectName: ObjectName, @RpcParam.Int value: Int)

  @RpcCall("setNetworkId")
  suspend fun setNetworkId(objectName: ObjectName, @RpcParam.UserType.NetworkId value: NetworkId)

  @RpcCall("setShowSearch")
  suspend fun setShowSearch(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("setSortAlphabetically")
  suspend fun setSortAlphabetically(objectName: ObjectName, @RpcParam.Bool value: Boolean)

  @RpcCall("update")
  suspend fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)

  //suspend fun update(objectName: ObjectName, properties: BufferViewConfigDto) = update(objectName, properties.let(BufferViewConfigDto.Serializer::serialize))
}
