/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.CoreInfoDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("CoreInfo", side = ProtocolSide.CORE)
interface CoreInfoClientApi {
  @RpcCall("setCoreData")
  suspend fun setCoreData(@RpcParam.QVariantMap data: QVariantMap)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: CoreInfoDto) = update(properties.let(CoreInfoDto.Serializer::serialize))
}
