/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.HighlightRuleManagerDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("HighlightRuleManager", side = ProtocolSide.CORE)
interface HighlightRuleManagerClientApi {

  @RpcCall("removeHighlightRule")
  suspend fun removeHighlightRule(@RpcParam.Int highlightRule: Int)

  @RpcCall("toggleHighlightRule")
  suspend fun toggleHighlightRule(@RpcParam.Int highlightRule: Int)

  @RpcCall("addHighlightRule")
  suspend fun addHighlightRule(
    @RpcParam.Int id: Int,
    @RpcParam.QString content: String?,
    @RpcParam.Bool isRegEx: Boolean,
    @RpcParam.Bool isCaseSensitive: Boolean,
    @RpcParam.Bool isEnabled: Boolean,
    @RpcParam.Bool isInverse: Boolean,
    @RpcParam.QString sender: String?,
    @RpcParam.QString channel: String?
  )

  @RpcCall("setHighlightNick")
  suspend fun setHighlightNick(@RpcParam.Int highlightNick: Int)

  @RpcCall("setNicksCaseSensitive")
  suspend fun setNicksCaseSensitive(@RpcParam.Bool nicksCaseSensitive: Boolean)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: HighlightRuleManagerDto) = update(properties.let(HighlightRuleManagerDto.Serializer::serialize))
}
