/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.dto.IdentityDto
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("Identity", side = ProtocolSide.CORE)
interface IdentityClientApi {
  @RpcCall("setAutoAwayEnabled")
  suspend fun setAutoAwayEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAutoAwayReason")
  suspend fun setAutoAwayReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setAutoAwayReasonEnabled")
  suspend fun setAutoAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAutoAwayTime")
  suspend fun setAutoAwayTime(objectName: ObjectName, @RpcParam.Int time: Int)


  @RpcCall("setAwayNick")
  suspend fun setAwayNick(objectName: ObjectName, @RpcParam.QString awayNick: String?)


  @RpcCall("setAwayNickEnabled")
  suspend fun setAwayNickEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setAwayReason")
  suspend fun setAwayReason(objectName: ObjectName, @RpcParam.QString awayReason: String?)


  @RpcCall("setAwayReasonEnabled")
  suspend fun setAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setDetachAwayEnabled")
  suspend fun setDetachAwayEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setDetachAwayReason")
  suspend fun setDetachAwayReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setDetachAwayReasonEnabled")
  suspend fun setDetachAwayReasonEnabled(objectName: ObjectName, @RpcParam.Bool enabled: Boolean)


  @RpcCall("setId")
  suspend fun setId(objectName: ObjectName, @RpcParam.UserType.IdentityId id: IdentityId)


  @RpcCall("setIdent")
  suspend fun setIdent(objectName: ObjectName, @RpcParam.QString ident: String?)


  @RpcCall("setIdentityName")
  suspend fun setIdentityName(objectName: ObjectName, @RpcParam.QString name: String?)


  @RpcCall("setKickReason")
  suspend fun setKickReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setNicks")
  suspend fun setNicks(objectName: ObjectName, @RpcParam.QStringList nicks: QStringList)


  @RpcCall("setPartReason")
  suspend fun setPartReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setQuitReason")
  suspend fun setQuitReason(objectName: ObjectName, @RpcParam.QString reason: String?)


  @RpcCall("setRealName")
  suspend fun setRealName(objectName: ObjectName, @RpcParam.QString realName: String?)


  @RpcCall("update")
  suspend fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(objectName: ObjectName, properties: IdentityDto) = update(objectName, properties.let(IdentityDto.Serializer::serialize))
}
