/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.IgnoreListManagerDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("IgnoreListManager", side = ProtocolSide.CORE)
interface IgnoreListManagerClientApi {
  @RpcCall("addIgnoreListItem")
  suspend fun addIgnoreListItem(
    @RpcParam.Int type: Int,
    @RpcParam.QString ignoreRule: String?,
    @RpcParam.Bool isRegEx: Boolean,
    @RpcParam.Int strictness: Int,
    @RpcParam.Int scope: Int,
    @RpcParam.QString scopeRule: String?,
    @RpcParam.Bool isActive: Boolean
  )

  @RpcCall("removeIgnoreListItem")
  suspend fun removeIgnoreListItem(@RpcParam.QString ignoreRule: String?)

  @RpcCall("requestToggleIgnoreRule")
  suspend fun toggleIgnoreRule(@RpcParam.QString ignoreRule: String?)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: IgnoreListManagerDto) = update(properties.let(IgnoreListManagerDto.Serializer::serialize))
}
