/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.dto.IrcChannelDto
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("IrcChannel", side = ProtocolSide.CORE)
interface IrcChannelClientApi  {
  @RpcCall("addChannelMode")
  suspend fun addChannelMode( objectName: ObjectName, @RpcParam.QChar mode: Char, @RpcParam.QString value: String? = null)


  @RpcCall("addUserMode")
  suspend fun addUserMode(objectName: ObjectName, @RpcParam.QString nick: String, @RpcParam.QString mode: String? = null)


  @RpcCall("joinIrcUsers")
  suspend fun joinIrcUsers(objectName: ObjectName, @RpcParam.QStringList nicks: QStringList, @RpcParam.QStringList modes: QStringList)


  @RpcCall("part")
  suspend fun part(objectName: ObjectName, @RpcParam.QString nick: String)


  @RpcCall("removeChannelMode")
  suspend fun removeChannelMode(objectName: ObjectName, @RpcParam.QChar mode: Char, @RpcParam.QString value: String? = null)


  @RpcCall("removeUserMode")
  suspend fun removeUserMode(objectName: ObjectName, @RpcParam.QString nick: String, @RpcParam.QString mode: String? = null)


  @RpcCall("setEncrypted")
  suspend fun setEncrypted(objectName: ObjectName, @RpcParam.Bool encrypted: Boolean)


  @RpcCall("setPassword")
  suspend fun setPassword(objectName: ObjectName, @RpcParam.QString password: String)


  @RpcCall("setTopic")
  suspend fun setTopic(objectName: ObjectName, @RpcParam.QString topic: String)


  @RpcCall("setUserModes")
  suspend fun setUserModes(objectName: ObjectName, @RpcParam.QString nick: String, @RpcParam.QString modes: String? = null)


  @RpcCall("update")
  suspend fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(objectName: ObjectName, properties: IrcChannelDto) = update(objectName, properties.let(IrcChannelDto.Serializer::serialize))
}
