/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.IrcListHelperDto
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.variant.QVariantList
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("IrcListHelper", side = ProtocolSide.CORE)
interface IrcListHelperClientApi  {
  @RpcCall("receiveChannelList")
  suspend fun receiveChannelList(@RpcParam.UserType.NetworkId netId: NetworkId, @RpcParam.QStringList channelFilters: QStringList, @RpcParam.QVariantList channels: QVariantList)

  @RpcCall("reportError")
  suspend fun reportError(@RpcParam.QString error: String?)

  @RpcCall("reportFinishedList")
  suspend fun reportFinishedList(@RpcParam.UserType.NetworkId netId: NetworkId)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: IrcListHelperDto) = update(properties.let(IrcListHelperDto.Serializer::serialize))
}
