/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.dto.IrcUserDto
import de.justjanne.libquassel.protocol.variant.QVariantMap
import org.threeten.bp.temporal.Temporal

@RpcApi("IrcUser", side = ProtocolSide.CORE)
interface IrcUserClientApi {
  @RpcCall("addUserModes")
  suspend fun addUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("joinChannel")
  suspend fun joinChannel(objectName: ObjectName, @RpcParam.QString channelname: String)

  @RpcCall("partChannel")
  suspend fun partChannel(objectName: ObjectName, @RpcParam.QString channelname: String)

  @RpcCall("quit")
  suspend fun quit(objectName: ObjectName)

  @RpcCall("removeUserModes")
  suspend fun removeUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("setAccount")
  suspend fun setAccount(objectName: ObjectName, @RpcParam.QString account: String)

  @RpcCall("setAway")
  suspend fun setAway(objectName: ObjectName, @RpcParam.Bool away: Boolean)

  @RpcCall("setAwayMessage")
  suspend fun setAwayMessage(objectName: ObjectName, @RpcParam.QString awayMessage: String)

  @RpcCall("setEncrypted")
  suspend fun setEncrypted(objectName: ObjectName, @RpcParam.Bool encrypted: Boolean)

  @RpcCall("setHost")
  suspend fun setHost(objectName: ObjectName, @RpcParam.QString host: String)

  @RpcCall("setIdleTime")
  suspend fun setIdleTime(objectName: ObjectName, @RpcParam.QDateTime idleTime: Temporal)

  @RpcCall("setIrcOperator")
  suspend fun setIrcOperator(objectName: ObjectName, @RpcParam.QString ircOperator: String)

  @RpcCall("setLastAwayMessage")
  suspend fun setLastAwayMessage(objectName: ObjectName, @RpcParam.Int lastAwayMessage: Int)

  @RpcCall("setLastAwayMessageTime")
  suspend fun setLastAwayMessageTime(objectName: ObjectName, @RpcParam.QDateTime lastAwayMessageTime: Temporal)

  @RpcCall("setLoginTime")
  suspend fun setLoginTime(objectName: ObjectName, @RpcParam.QDateTime loginTime: Temporal)

  @RpcCall("setNick")
  suspend fun setNick(objectName: ObjectName, @RpcParam.QString newNick: String)

  @RpcCall("setRealName")
  suspend fun setRealName(objectName: ObjectName, @RpcParam.QString realName: String)

  @RpcCall("setServer")
  suspend fun setServer(objectName: ObjectName, @RpcParam.QString server: String)

  @RpcCall("setSuserHost")
  suspend fun setSuserHost(objectName: ObjectName, @RpcParam.QString suserHost: String)

  @RpcCall("setUser")
  suspend fun setUser(objectName: ObjectName, @RpcParam.QString user: String)

  @RpcCall("setUserModes")
  suspend fun setUserModes(objectName: ObjectName, @RpcParam.QString modes: String)

  @RpcCall("setWhoisServiceReply")
  suspend fun setWhoisServiceReply(objectName: ObjectName, @RpcParam.QString whoisServiceReply: String)

  @RpcCall("updateHostmask")
  suspend fun updateHostmask(objectName: ObjectName, @RpcParam.QString mask: String)

  @RpcCall("update")
  suspend fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(objectName: ObjectName, properties: IrcUserDto) = update(objectName, properties.let(IrcUserDto.Serializer::serialize))
}
