/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.variant.QVariantList
import de.justjanne.libquassel.protocol.variant.QVariantMap
import java.nio.ByteBuffer

@RpcApi("Network", side = ProtocolSide.CORE)
interface NetworkClientApi {
  @RpcCall("setNetworkName")
  suspend fun setNetworkName(objectName: ObjectName, @RpcParam.QString networkName: String)


  @RpcCall("setCurrentServer")
  suspend fun setCurrentServer(objectName: ObjectName, @RpcParam.QString currentServer: String?)


  @RpcCall("setMyNick")
  suspend fun setMyNick(objectName: ObjectName, @RpcParam.QString myNick: String?)


  @RpcCall("setLatency")
  suspend fun setLatency(objectName: ObjectName, @RpcParam.Int latency: Int)


  @RpcCall("setCodecForServer")
  suspend fun setCodecForServer(objectName: ObjectName, @RpcParam.QByteArray codecForServer: ByteBuffer)


  @RpcCall("setCodecForEncoding")
  suspend fun setCodecForEncoding(objectName: ObjectName, @RpcParam.QByteArray codecForEncoding: ByteBuffer)


  @RpcCall("setCodecForDecoding")
  suspend fun setCodecForDecoding(objectName: ObjectName, @RpcParam.QByteArray codecForDecoding: ByteBuffer)


  @RpcCall("setIdentity")
  suspend fun setIdentity(objectName: ObjectName, @RpcParam.UserType.IdentityId identityId: IdentityId)


  @RpcCall("setConnected")
  suspend fun setConnected(objectName: ObjectName, @RpcParam.Bool isConnected: Boolean)


  @RpcCall("setConnectionState")
  suspend fun setConnectionState(objectName: ObjectName, @RpcParam.Int connectionState: Int)


  @RpcCall("setUseRandomServer")
  suspend fun setUseRandomServer(objectName: ObjectName, @RpcParam.Bool useRandomServer: Boolean)


  @RpcCall("setPerform")
  suspend fun setPerform(objectName: ObjectName, @RpcParam.QStringList perform: QStringList)


  @RpcCall("setSkipCaps")
  suspend fun setSkipCaps(objectName: ObjectName, @RpcParam.QStringList skipCaps: QStringList)


  @RpcCall("setUseAutoIdentify")
  suspend fun setUseAutoIdentify(objectName: ObjectName, @RpcParam.Bool useAutoIdentify: Boolean)


  @RpcCall("setAutoIdentifyService")
  suspend fun setAutoIdentifyService(objectName: ObjectName, @RpcParam.QString autoIdentifyService: String)


  @RpcCall("setAutoIdentifyPassword")
  suspend fun setAutoIdentifyPassword(objectName: ObjectName, @RpcParam.QString autoIdentifyPassword: String)


  @RpcCall("setUseSasl")
  suspend fun setUseSasl(objectName: ObjectName, @RpcParam.Bool useSasl: Boolean)


  @RpcCall("setSaslAccount")
  suspend fun setSaslAccount(objectName: ObjectName, @RpcParam.QString saslAccount: String)


  @RpcCall("setSaslPassword")
  suspend fun setSaslPassword(objectName: ObjectName, @RpcParam.QString saslPassword: String)


  @RpcCall("setUseAutoReconnect")
  suspend fun setUseAutoReconnect(objectName: ObjectName, @RpcParam.Bool useAutoReconnect: Boolean)


  @RpcCall("setAutoReconnectInterval")
  suspend fun setAutoReconnectInterval(objectName: ObjectName, @RpcParam.UInt autoReconnectInterval: UInt)


  @RpcCall("setAutoReconnectRetries")
  suspend fun setAutoReconnectRetries(objectName: ObjectName, @RpcParam.UShort autoReconnectRetries: UShort)


  @RpcCall("setUnlimitedReconnectRetries")
  suspend fun setUnlimitedReconnectRetries(objectName: ObjectName, @RpcParam.Bool unlimitedReconnectRetries: Boolean)


  @RpcCall("setRejoinChannels")
  suspend fun setRejoinChannels(objectName: ObjectName, @RpcParam.Bool rejoinChannels: Boolean)


  @RpcCall("setUseCustomMessageRate")
  suspend fun setUseCustomMessageRate(objectName: ObjectName, @RpcParam.Bool useCustomMessageRate: Boolean)


  @RpcCall("setMessageRateBurstSize")
  suspend fun setMessageRateBurstSize(objectName: ObjectName, @RpcParam.UInt messageRateBurstSize: UInt)


  @RpcCall("setMessageRateDelay")
  suspend fun setMessageRateDelay(objectName: ObjectName, @RpcParam.UInt messageRateDelay: UInt)


  @RpcCall("setUnlimitedMessageRate")
  suspend fun setUnlimitedMessageRate(objectName: ObjectName, @RpcParam.Bool unlimitedMessageRate: Boolean)


  @RpcCall("setServerList")
  suspend fun setServerList(objectName: ObjectName, @RpcParam.QVariantList serverList: QVariantList)


  @RpcCall("addSupport")
  suspend fun addSupport(objectName: ObjectName, @RpcParam.QString param: String, @RpcParam.QString value: String = "")


  @RpcCall("removeSupport")
  suspend fun removeSupport(objectName: ObjectName, @RpcParam.QString param: String)


  @RpcCall("addCap")
  suspend fun addCap(objectName: ObjectName, @RpcParam.QString capability: String, @RpcParam.QString value: String = "")


  @RpcCall("acknowledgeCap")
  suspend fun acknowledgeCap(objectName: ObjectName, @RpcParam.QString capability: String)


  @RpcCall("removeCap")
  suspend fun removeCap(objectName: ObjectName, @RpcParam.QString capability: String)


  @RpcCall("clearCaps")
  suspend fun clearCaps(objectName: ObjectName)


  @RpcCall("addIrcUser")
  suspend fun addIrcUser(objectName: ObjectName, @RpcParam.QString hostmask: String)


  @RpcCall("addIrcChannel")
  suspend fun addIrcChannel(objectName: ObjectName, @RpcParam.QString channel: String)

  @RpcCall("update")
  suspend fun update(objectName: ObjectName, @RpcParam.QVariantMap properties: QVariantMap)

  //suspend fun update(objectName: ObjectName, properties: NetworkDto) = update(objectName, properties.let(NetworkDto.Serializer::serialize))
}
