/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.NetworkConfigDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("NetworkConfig", side = ProtocolSide.CORE)
interface NetworkConfigClientApi {
  @RpcCall("setAutoWhoDelay")
  suspend fun setAutoWhoDelay(@RpcParam.Int delay: Int)

  @RpcCall("setAutoWhoEnabled")
  suspend fun setAutoWhoEnabled(@RpcParam.Bool enabled: Boolean)

  @RpcCall("setAutoWhoInterval")
  suspend fun setAutoWhoInterval(@RpcParam.Int interval: Int)

  @RpcCall("setAutoWhoNickLimit")
  suspend fun setAutoWhoNickLimit(@RpcParam.Int limit: Int)

  @RpcCall("setMaxPingCount")
  suspend fun setMaxPingCount(@RpcParam.Int count: Int)

  @RpcCall("setPingInterval")
  suspend fun setPingInterval(@RpcParam.Int interval: Int)

  @RpcCall("setPingTimeoutEnabled")
  suspend fun setPingTimeoutEnabled(@RpcParam.Bool enabled: Boolean)

  @RpcCall("setStandardCtcp")
  suspend fun setStandardCtcp(@RpcParam.Bool enabled: Boolean)

  @RpcCall("update")
  suspend fun update(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun update(properties: NetworkConfigDto) = update(properties.let(NetworkConfigDto.Serializer::serialize))
}
