/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.client

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.models.BufferInfo
import de.justjanne.libquassel.protocol.models.Message
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.variant.QVariantMap
import java.nio.ByteBuffer

@RpcApi(side = ProtocolSide.CORE)
interface RpcClientApi {
  @RpcCall("__objectRenamed__")
  suspend fun objectRenamed(@RpcParam.QByteArray classname: ByteBuffer, @RpcParam.QString newName: String?, @RpcParam.QString oldName: String?)

  @RpcCall("2displayMsg(Message)")
  suspend fun displayMsg(@RpcParam.UserType.Message message: Message)

  @RpcCall("2displayStatusMsg(QString,QString)")
  suspend fun displayStatusMsg(@RpcParam.QString net: String?, @RpcParam.QString msg: String?)

  @RpcCall("2bufferInfoUpdated(BufferInfo)")
  suspend fun bufferInfoUpdated(@RpcParam.UserType.BufferInfo bufferInfo: BufferInfo)

  @RpcCall("2identityCreated(Identity)")
  suspend fun identityCreated(@RpcParam.QVariantMap identity: QVariantMap)

  @RpcCall("2identityRemoved(IdentityId)")
  suspend fun identityRemoved(@RpcParam.UserType.IdentityId identityId: IdentityId)

  @RpcCall("2networkCreated(NetworkId)")
  suspend fun networkCreated(@RpcParam.UserType.NetworkId networkId: NetworkId)

  @RpcCall("2networkRemoved(NetworkId)")
  suspend fun networkRemoved(@RpcParam.UserType.NetworkId networkId: NetworkId)

  @RpcCall("2passwordChanged(PeerPtr,bool)")
  suspend fun passwordChanged(@RpcParam.UserType.PeerPtr peer: ULong, @RpcParam.Bool success: Boolean)

  @RpcCall("2disconnectFromCore()")
  suspend fun disconnectFromCore()
}
