/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.AliasManagerDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("AliasManager", side = ProtocolSide.CLIENT)
interface AliasManagerServerApi {
  @RpcCall("addAlias")
  suspend fun addAlias(
    @RpcParam.QString name: String,
    @RpcParam.QString expansion: String
  )

  @RpcCall("requestUpdate")
  suspend fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun requestUpdate(properties: AliasManagerDto) = requestUpdate(properties.let(AliasManagerDto.Serializer::serialize))
}
