/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.HighlightRuleManagerDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("HighlightRuleManager", side = ProtocolSide.CLIENT)
interface HighlightRuleManagerServerApi {
  @RpcCall("requestRemoveHighlightRule")
  suspend fun requestRemoveHighlightRule(@RpcParam.Int highlightRule: Int)

  @RpcCall("requestToggleHighlightRule")
  suspend fun requestToggleHighlightRule(@RpcParam.Int highlightRule: Int)

  @RpcCall("requestAddHighlightRule")
  suspend fun requestAddHighlightRule(
    @RpcParam.Int id: Int,
    @RpcParam.QString content: String?,
    @RpcParam.Bool isRegEx: Boolean,
    @RpcParam.Bool isCaseSensitive: Boolean,
    @RpcParam.Bool isEnabled: Boolean,
    @RpcParam.Bool isInverse: Boolean,
    @RpcParam.QString sender: String?,
    @RpcParam.QString channel: String?
  )

  @RpcCall(
    "requestSetHighlightNick")
  suspend fun requestSetHighlightNick(@RpcParam.Int highlightNick: Int)

  @RpcCall("requestSetNicksCaseSensitive")
  suspend fun requestSetNicksCaseSensitive(@RpcParam.Bool nicksCaseSensitive: Boolean)

  @RpcCall("requestUpdate")
  suspend fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun requestUpdate(properties: HighlightRuleManagerDto) = requestUpdate(properties.let(HighlightRuleManagerDto.Serializer::serialize))
}
