/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.IrcListHelperDto
import de.justjanne.libquassel.protocol.models.QStringList
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("IrcListHelper", side = ProtocolSide.CLIENT)
interface IrcListHelperServerApi {
  @RpcCall("requestChannelList")
  suspend fun requestChannelList(@RpcParam.UserType.NetworkId netId: NetworkId, @RpcParam.QStringList channelFilters: QStringList)

  @RpcCall("requestUpdate")
  suspend fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun requestUpdate(properties: IrcListHelperDto) = requestUpdate(properties.let(IrcListHelperDto.Serializer::serialize))
}
