/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.api.server

import de.justjanne.libquassel.annotations.ProtocolSide
import de.justjanne.libquassel.annotations.RpcApi
import de.justjanne.libquassel.annotations.RpcCall
import de.justjanne.libquassel.annotations.RpcParam
import de.justjanne.libquassel.protocol.dto.NetworkConfigDto
import de.justjanne.libquassel.protocol.variant.QVariantMap

@RpcApi("NetworkConfig", side = ProtocolSide.CLIENT)
interface NetworkConfigServerApi {
  @RpcCall("requestSetAutoWhoDelay")
  suspend fun requestSetAutoWhoDelay(@RpcParam.Int delay: Int)

  @RpcCall("requestSetAutoWhoEnabled")
  suspend fun requestSetAutoWhoEnabled(@RpcParam.Bool enabled: Boolean)

  @RpcCall("requestSetAutoWhoInterval")
  suspend fun requestSetAutoWhoInterval(@RpcParam.Int interval: Int)

  @RpcCall("requestSetAutoWhoNickLimit")
  suspend fun requestSetAutoWhoNickLimit(@RpcParam.Int limit: Int)

  @RpcCall("requestSetMaxPingCount")
  suspend fun requestSetMaxPingCount(@RpcParam.Int count: Int)

  @RpcCall("requestSetPingInterval")
  suspend fun requestSetPingInterval(@RpcParam.Int interval: Int)

  @RpcCall("requestSetPingTimeoutEnabled")
  suspend fun requestSetPingTimeoutEnabled(@RpcParam.Bool enabled: Boolean)

  @RpcCall("requestSetStandardCtcp")
  suspend fun requestSetStandardCtcp(@RpcParam.Bool enabled: Boolean)

  @RpcCall("requestUpdate")
  suspend fun requestUpdate(@RpcParam.QVariantMap properties: QVariantMap)

  suspend fun requestUpdate(properties: NetworkConfigDto) = requestUpdate(properties.let(NetworkConfigDto.Serializer::serialize))
}
